@@ -41,10 +41,17 @@
 #endif
 
 
-enum {TT_UNKNOWN_CFG, TT_SOURCE, TT_SELECT, TT_UNSELECT, TT_EXPORT, TT_SETUP3D};
+enum {TT_UNKNOWN_CFG, 
+TT_SOURCE, 
+TT_SELECT, 
+TT_UNSELECT, 
+TT_EXPORT, 
+TT_SETUP3D,
+TT_AUTO_JOIN};
 
 
 static const thstok thtt_cfg[] = {
+//  {"auto-join", TT_AUTO_JOIN},
   {"export", TT_EXPORT},
   {"select", TT_SELECT},
   {"setup3d", TT_SETUP3D},
@@ -66,6 +73,8 @@ thconfig::thconfig()
   this->install_tex = false;
   this->install_tcltk = false;
 
+  this->auto_join = true;
+
   this->tmp3dSMP = 1.0;
   this->tmp3dWALLSMP = 1.0;
   this->tmp3dMAXDIMD = 5.0;
@@ -346,6 +355,17 @@ void thconfig::load()
                 ththrow(("invalid number -- %s", valuemb.get_buffer()[2]))
             }
             break;
+
+          case TT_AUTO_JOIN:
+            if (valuemb.get_size() > 0) {
+              sv = thmatch_token(valuemb.get_buffer()[0], thtt_bool);
+              if (sv == TT_UNKNOWN_BOOL)
+                ththrow(("invalid auto-join sqitch -- %s", valuemb.get_buffer()[0]))
+              this->auto_join = (sv == TT_TRUE);
+            } else {
+              ththrow(("missing auto-join switch"))
+            }
+            break;
   
           case TT_SELECT:
             this->selector.parse_selection(false, valuemb.get_size(), valuemb.get_buffer());
@@ -614,9 +634,18 @@ void thconfig::export_data()
 void thconfig::xth_save()
 {
   if (this->generate_xthcfg) {
+
+    thbuffer nfn;
+    nfn = this->fname.get_buffer();
+    char * xbf = nfn.get_buffer();
+    size_t x, xl = strlen(xbf);
+    for (x = 0; x < xl; x++) {
+      if (xbf[x] == '.')
+        xbf[x] = '_';
+    }
   
 #ifdef THDEBUG
-    thprintf("\nwriting xtherion file -- .xth-%s\n", this->fname.get_buffer());
+    thprintf("\nwriting xtherion file -- .xth_%s\n", xbf);
 #else
     thprintf("writing xtherion file ... ");
     thtext_inline = true;
@@ -624,8 +653,9 @@ void thconfig::xth_save()
 
     // OK, let's open configuration file for output
     FILE * cf;
-    this->dbptr->buff_tmp = ".xth-";
-    this->dbptr->buff_tmp += this->fname.get_buffer();
+    this->dbptr->buff_tmp = ".xth_";
+    this->dbptr->buff_tmp += xbf;
+    this->dbptr->buff_tmp += "_xth";
     cf = fopen(this->dbptr->buff_tmp.get_buffer(),"w");
     if (cf == NULL) {
       thwarning(("can't open xtherion file for output -- %s.xth", this->fname.get_buffer()));
