@@ -40,7 +40,15 @@
 #include "thconfig.h"
 #include <stdio.h>
 #include "thtmpdir.h"
+#ifndef THMSVC
 #include <unistd.h>
+#else
+#include <direct.h>
+#define getcwd _getcwd
+#define chdir _chdir
+#define putenv _putenv
+#define hypot _hypot
+#endif
 #include "thchenc.h"
 #include "thdb1d.h"
 #include "thinit.h"
@@ -49,19 +57,24 @@
 #include "thpdf.h"
 #include "thpdfdata.h"
 #include "thmpost.h"
-#include "thtex.h"
+#include "thtex.h"   
 #include "thcmdline.h"
 #include "thtexfonts.h"
 #include "thsurvey.h"
+#include "thlegenddata.h"
 #include <fstream>
 #include <map>
 #include <set>
 #include "thmapstat.h"
 #include "thsurface.h"
 #include <stdlib.h>
+#include "extern/lxMath.h"
+#include "thsvg.h"
+#include "extern/img.h"
 
-#ifdef THWIN32
+#ifdef THMSVC
 #define snprintf _snprintf
+#define strcasecmp _stricmp
 #endif
 
 thexpmap::thexpmap() {
@@ -151,6 +164,10 @@ void thexpmap::parse_options(int & argx, int nargs, char ** args)
         case TT_EXP_MAP:
           switch (this->format) {
             case TT_EXPMAP_FMT_PDF:
+            case TT_EXPMAP_FMT_SVG:
+            case TT_EXPMAP_FMT_XHTML:
+            case TT_EXPMAP_FMT_XVI:
+            case TT_EXPMAP_FMT_3D:
               supform = true;
           }
           break;
@@ -234,7 +251,7 @@ void thexpmap::dump_body(FILE * xf)
   if (strcmp(this->projstr,"plan") != 0)
     fprintf(xf," -projection %s",this->projstr);
   thexport::dump_body(xf);
-  if (this->format != this->get_default_format())
+  if (this->format != TT_EXPMAP_FMT_UNKNOWN)
     fprintf(xf," -format %s", thmatch_string(this->format, thtt_expmap_fmt));
   thdecode(&(this->cfgptr->bf1), this->cfgptr->cfg_fenc, this->layoutopts.get_buffer());
   fprintf(xf,"%s",this->cfgptr->bf1.get_buffer());
@@ -261,30 +278,34 @@ void thexpmap::process_db(class thdatabase * dbp)
   if (this->projptr == NULL)
     return;
 
-  if (this->format == TT_EXPMAP_FMT_UNKNOWN)
-    this->format = this->get_default_format();
+  if (this->format == TT_EXPMAP_FMT_UNKNOWN) {
+    this->format = TT_EXPMAP_FMT_PDF;
+    thexp_set_ext_fmt(".pdf", TT_EXPMAP_FMT_PDF)
+    if (this->export_mode == TT_EXP_MAP) {
+      thexp_set_ext_fmt(".svg", TT_EXPMAP_FMT_SVG)
+      thexp_set_ext_fmt(".xhtml", TT_EXPMAP_FMT_XHTML)
+      thexp_set_ext_fmt(".xvi", TT_EXPMAP_FMT_XVI)
+      thexp_set_ext_fmt(".3d", TT_EXPMAP_FMT_3D)
+    }
+  }
 
-  thdb.db2d.process_projection(this->projptr);
-/*  if (this->projptr->first_scrap == NULL) {
-    thwarning(("%s [%d] -- no projection data -- %s",
-      this->src.name, this->src.line, this->projstr))
-    return;
-  }*/
-  
-  // parse layout
-  // thlayout * tmp = NULL;
-  // this->layout = dbp->get_layout(this->layoutstr);
-  // if (this->layout == NULL) {
-  //  tmp = new thlayout;
-  //  this->layout = tmp;
-  // }
+  if (this->format != TT_EXPMAP_FMT_XVI)
+    thdb.db2d.process_projection(this->projptr);
   this->layout->process_copy();
   
   // the export it self
   switch (this->format) {
+    case TT_EXPMAP_FMT_3D:
+      this->export_uni(thdb.db2d.select_projection(this->projptr),this->projptr);
+      break;
     case TT_EXPMAP_FMT_PDF:
+    case TT_EXPMAP_FMT_XHTML:
+    case TT_EXPMAP_FMT_SVG:
       this->export_pdf(thdb.db2d.select_projection(this->projptr),this->projptr);
       break;
+    case TT_EXPMAP_FMT_XVI:
+      this->export_xvi(this->projptr);
+      break;
   }
   
   //if (tmp != NULL) {
@@ -292,12 +313,6 @@ void thexpmap::process_db(class thdatabase * dbp)
   //}
 }
 
-int thexpmap::get_default_format() {
-  if (this->export_mode == TT_EXP_MAP)
-    return TT_EXPMAP_FMT_PDF;
-  else 
-    return TT_EXPMAP_FMT_PDF;
-}
 
 
 char * thexpmap_u2string(unsigned u) {
@@ -338,6 +353,7 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
   bool export_sections, export_outlines_only;
   double shx, shy;
   unsigned long sclevel, bmlevel;
+  legenddata ldata;
 
   bool anyprev, anyprevabove = false, anyprevbelow = false;
   char * prevbf;
@@ -374,8 +390,11 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
   // scalebar length
   double sblen;
   int sbtest;
+
+	this->layout->units.lang = this->layout->lang;
+
   if (this->layout->scale_bar <= 0.0) {
-    sbtest = int(log(0.10 / this->layout->scale) / log(pow(10.0,1.0/3.0)));
+    sbtest = int(log(this->layout->units.convert_length(0.10 / this->layout->scale)) / log(pow(10.0,1.0/3.0)));
     sblen = pow(10.0,double(sbtest/3));
     switch (sbtest % 3) {
       case 0: sblen *= 1.0; break;
@@ -383,7 +402,8 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
       default: sblen *= 5.0; break;
     }
   } else 
-    sblen = this->layout->scale_bar;
+    sblen = this->layout->units.convert_length(this->layout->scale_bar);
+		
 
 #define layoutnan(XXX,VVV) \
   if (thisnan(this->layout->XXX)) this->layout->XXX = (VVV)    
@@ -416,9 +436,24 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
     origin_shy = 0.0;
   }
   // koniec korekcie shiftu na layout origin
+
+	if ((this->layout->def_overlap) && (!this->layout->def_size)) {
+		double nhs, nvs;
+		nhs = this->layout->hsize + 0.02 - 2.0 * this->layout->overlap;
+		nvs = this->layout->vsize + 0.02 - 2.0 * this->layout->overlap;
+		if (nhs > 0.0)
+			this->layout->hsize = nhs;
+		if (nvs > 0.0)
+			this->layout->vsize = nvs;
+	} else if ((this->layout->def_size) && (!this->layout->def_page_setup)) {
+	  this->layout->paghs = this->layout->hsize + 2.0 * this->layout->overlap;
+	  this->layout->pagvs = this->layout->vsize + 2.0 * this->layout->overlap + 0.065;
+	  this->layout->paphs = this->layout->paghs + 2.0 * this->layout->marls;
+	  this->layout->papvs = this->layout->pagvs + 2.0 * this->layout->marts;
+	}
       
 //  tf = fopen(thtmp.get_file_name("Config"),"w");
-//  this->layout->export_config(tf,prj,out.ms,origin_shx,origin_shy);     
+//  this->layout->export_config(tf,prj,out.ms,origin_shx,origin_shy);
   this->layout->set_thpdf_layout(prj,out.ms,origin_shx,origin_shy);
 //  fclose(tf);
 
@@ -511,7 +546,7 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
     fprintf(plf,"# COLOR LEGEND\n");
     for (list<colorlegendrecord>::iterator cli = COLORLEGENDLIST.begin();
       cli != COLORLEGENDLIST.end(); cli++) {
-      fprintf(plf,"# %4.0f %4.0f %4.0f %s\n", 100.0 * cli->R, 100.0 * cli->G, 100.0 * cli->B, cli->name.c_str());
+      fprintf(plf,"# %4.0f %4.0f %4.0f %s\n", 100.0 * cli->R, 100.0 * cli->G, 100.0 * cli->B, cli->texname.c_str());
     }
     fprintf(plf,"\n\n\n");
   }
@@ -800,9 +835,9 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
 
   sprintf(texb.get_buffer(),"data.%d",sfig);
   LAYOUT.scalebar = texb.get_buffer();
-  snprintf(prevbf,127,"%g",sblen);
-  fprintf(mpf,"beginfig(%d);\ns_scalebar(%g, 1.0, \"%s\");\nendfig;\n",
-    sfig++, sblen,  utf2tex(thT("units m",layout->lang)));
+  //snprintf(prevbf,127,"%g",sblen);
+  fprintf(mpf,"beginfig(%d);\ns_scalebar(%g, %g, \"%s\");\nendfig;\n",
+    sfig++, sblen, 1.0 / this->layout->units.convert_length(1.0), utf2tex(this->layout->units.format_i18n_length_units()));
 
   // sem pride zapisanie legendy do MP suboru
   if (this->layout->def_base_scale || this->layout->redef_base_scale)
@@ -856,6 +891,17 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
           srfpr.xy = surf->calib_xy * surfscl * surf->pict_dpi / 300.0;
           srfpr.yx = surf->calib_yx * surfscl * surf->pict_dpi / 300.0;
           srfpr.yy = surf->calib_yy * surfscl * surf->pict_dpi / 300.0;
+          srfpr.width = surf->pict_width / surf->pict_dpi * 72.0;
+          srfpr.height = surf->pict_height / surf->pict_dpi * 72.0;
+          srfpr.type = "unknown";
+          switch (surf->pict_type) {
+            case TT_IMG_TYPE_JPEG:
+              srfpr.type = "jpeg";
+              break;
+            case TT_IMG_TYPE_PNG:
+              srfpr.type = "png";
+              break;
+          }
 					
 					// otocenie o rotaciu v layoute
 					double tdx = srfpr.dx, tdy = srfpr.dy,
@@ -1095,35 +1141,48 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
     
   tf = fopen(thtmp.get_file_name("th_texts.tex"),"w");
   fprintf(tf,"\\legendtitle={%s}\n",utf2tex(thT("title legend",this->layout->lang)));
+  ldata.legendtitle = thT("title legend",this->layout->lang);
 
-  if (this->layout->color_crit != TT_LAYOUT_CCRIT_UNKNOWN) {
+  ldata.colorlegendtitle = "";
+  if ((this->layout->color_crit != TT_LAYOUT_CCRIT_UNKNOWN) && (this->layout->color_crit != TT_LAYOUT_CCRIT_SCRAP)) {
     fprintf(tf,"\\colorlegendtitle={");
     switch (this->layout->color_crit) {
       case TT_LAYOUT_CCRIT_ALTITUDE:
         fprintf(tf,"%s", utf2tex(thT("title color-legend-altitude",this->layout->lang)));
+        ldata.colorlegendtitle = thT("title color-legend-altitude",this->layout->lang);
         break;
       default: //TT_LAYOUT_CCRIT_MAP:
         fprintf(tf,"%s", utf2tex(thT("title color-legend-map",this->layout->lang)));
+        ldata.colorlegendtitle = thT("title color-legend-map",this->layout->lang);
     }
     fprintf(tf,"}\n");
   }
 
   // ak neni atlas, tak nastavi legendcavename
   fprintf(tf,"\\cavename={%s}\n",utf2tex(tit.get_buffer()));
+  ldata.cavename = tit.get_buffer();
+  ldata.comment = "";
 
   if (strlen(this->layout->doc_comment) > 0) {
     fprintf(tf,"\\comment={%s}\n",utf2tex(this->layout->doc_comment));
+    ldata.comment = this->layout->doc_comment;
   }
   
-  if ((prj->type != TT_2DPROJ_PLAN) || (!this->symset.assigned[SYMS_NORTHARROW]))
+  if ((prj->type != TT_2DPROJ_PLAN) || (!this->symset.assigned[SYMS_NORTHARROW])) {
     fprintf(tf,"\\northarrowfalse\n");
-  else 
+    ldata.northarrow = false;
+  } else {
     fprintf(tf,"\\northarrowtrue\n");
-  if (!this->symset.assigned[SYMS_SCALEBAR])
+    ldata.northarrow = true;
+  }
+  if (!this->symset.assigned[SYMS_SCALEBAR]) {
     fprintf(tf,"\\scalebarfalse\n");
-  else 
+    ldata.scalebar = false;
+  } else {
     fprintf(tf,"\\scalebartrue\n");
-  prj->stat.export_pdftex(tf, this->layout);
+    ldata.scalebar = true;
+  }
+  prj->stat.export_pdftex(tf, this->layout, &ldata);
   fclose(tf);
 
   // teraz sa hodi do temp adresara - spusti metapost, thpdf, a pdftex a skopiruje vysledok
@@ -1146,7 +1205,11 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
     putenv("MFBASES=");
     putenv("MFINPUTS=");
     putenv("MFPOOL=");
+#ifdef THMSVC
+    putenv("MPINPUTS=../mpost;.");
+#else
     putenv("MPINPUTS=");
+#endif
     putenv("MPMEMS=");
     putenv("MPPOOL=");
     putenv("MPSUPPORT=");
@@ -1201,57 +1264,83 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
     com = "thpdf";
 */
 
-  if (!quick_map_exp)
-    thconvert();
 
-//  thpdf((this->export_mode == TT_EXP_MAP ? 1 : 0),wdir.get_buffer());
-  thpdf((this->export_mode == TT_EXP_MAP ? 1 : 0));
-  
-  com = "\"";
-  com += thini.get_path_pdftex();
-  com += "\"";
-//  com += " --interaction nonstopmode data.tex";
-  com += " data.tex";
-#ifdef THDEBUG
-  thprintf("running pdftex\n");
-#endif
-  retcode = system(com.get_buffer());
-  thexpmap_log_log_file("data.log",
-  "######################## pdftex log file #########################\n",
-  "##################### end of pdftex log file #####################\n",false);
-  if (retcode != EXIT_SUCCESS) {
-    chdir(wdir.get_buffer());
-    ththrow(("pdftex exit code -- %d", retcode))
-  }
-
-    
-  // Let's copy results and log-file to working directory
-  chdir(wdir.get_buffer());
 #ifdef THWIN32
-  com = "copy ";
+      unsigned long cpch;
+      char * cpcmd;
+#endif
+
+
+  switch (this->format) {
+
+    case TT_EXPMAP_FMT_PDF:
+      if (!quick_map_exp)
+        thconvert();
+
+      thpdf((this->export_mode == TT_EXP_MAP ? 1 : 0));
+
+      com = "\"";
+      com += thini.get_path_pdftex();
+      com += "\"";
+    //  com += " --interaction nonstopmode data.tex";
+      com += " data.tex";
+#ifdef THDEBUG
+      thprintf("running pdftex\n");
+#endif
+      retcode = system(com.get_buffer());
+      thexpmap_log_log_file("data.log",
+      "######################## pdftex log file #########################\n",
+      "##################### end of pdftex log file #####################\n",false);
+      if (retcode != EXIT_SUCCESS) {
+        chdir(wdir.get_buffer());
+        ththrow(("pdftex exit code -- %d", retcode))
+      }
+
+      // Let's copy results and log-file to working directory
+      chdir(wdir.get_buffer());
+#ifdef THWIN32
+      com = "copy ";
 #else
-  com = "cp ";
+      com = "cp ";
 #endif
-  com += thtmp.get_file_name("data.pdf");
-  com += " ";
-  com += fnm;
+      com += thtmp.get_file_name("data.pdf");
+      com += " ";
+      com += fnm;
 
 #ifdef THWIN32
-  unsigned long cpch;
-  char * cpcmd = com.get_buffer();
-  for(cpch = 0; cpch < strlen(cpcmd); cpch++) {
-    if (cpcmd[cpch] == '/') {
-      cpcmd[cpch] = '\\';
-    }
-  }
+      cpcmd = com.get_buffer();
+      for(cpch = 0; cpch < strlen(cpcmd); cpch++) {
+       if (cpcmd[cpch] == '/') {
+         cpcmd[cpch] = '\\';
+       }
+     }
 #endif
 
 #ifdef THDEBUG
-  thprintf("copying results\n");
+      thprintf("copying results\n");
 #endif
-  retcode = system(com.get_buffer());
-  if (retcode != EXIT_SUCCESS)
-    ththrow(("cp exit code -- %d", retcode))
+      retcode = system(com.get_buffer());
+      if (retcode != EXIT_SUCCESS)
+        ththrow(("cp exit code -- %d", retcode))
+      break;
+      // END OF PDF POSTPROCESSING
+
+    case TT_EXPMAP_FMT_SVG:
+      thconvert_new();
+      chdir(wdir.get_buffer());
+      thsvg(fnm, 0, ldata);
+      break;
+      
+
+    case TT_EXPMAP_FMT_XHTML:
+      thconvert_new();
+      chdir(wdir.get_buffer());
+      thsvg(fnm, 1, ldata);
+      break;
+
+  }
+
+
 #ifdef THDEBUG
 #else
   thprintf("done\n");
@@ -1260,11 +1349,25 @@ void thexpmap::export_pdf(thdb2dxm * maps, thdb2dprj * prj) {
   
 }
 
+struct vec2d {
+	double x;
+	double y;
+	vec2d() : x(0.0), y(0.0) {}
+	vec2d(double xx, double yy) : x(xx), y(yy) {}
+	void set(double xx, double yy);
+};
+
+void vec2d::set(double xx, double yy) {
+	this->x = xx;
+	this->y = yy;
+}
+
 
 thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap, 
     unsigned & startnum, bool outline_mode)
 {
   th2ddataobject * obj;
+  thpoint * ptp;
   thexpmapmpxs nooutc, * noout;
   nooutc = *out;
   noout = &(nooutc);
@@ -1276,6 +1379,41 @@ thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap,
   bool somex, first, vis;
   thscraplp * slp;
   thdb2dlp * lp;
+  thexception dbg_stnms;
+	
+	// check scrap limits
+  fprintf(out->file, "warningcheck := 1;\n");
+	if (!thisnan(scrap->lxmin)) {
+		vec2d v1, v2;
+		v1.set(thxmmxst(out, scrap->lxmin, scrap->lymin)); 
+		v2.set(thxmmxst(out, scrap->lxmax, scrap->lymax));
+		double max, minscale;
+		max = 0;
+#define mmmmcx(v) if (fabs(v) > max) max = fabs(v);
+		mmmmcx(v1.x);
+		mmmmcx(v1.y);
+		mmmmcx(v2.x);
+		mmmmcx(v2.y);
+		if (max > 4095.0) {
+		  minscale = (1.0 / out->layout->scale) * (max / 4095.0);
+			if (scrap->centerline_io) {
+			  fprintf(out->file, "warningcheck := 0;\n");
+				if (max > 32767.0) {
+				  minscale = (1.0 / out->layout->scale) * (max / 32767.0);
+					ththrow(("scale too large -- maximal scale for this configuration is 1 : %.0f", minscale + 1));
+				}
+			} else {
+				ththrow(("scrap %s@%s defined at %s [%d] is too large to process in metapost in this scale -- maximal scale for this scrap is 1 : %.0f", 
+					scrap->name, 
+					scrap->fsptr->full_name, 
+					scrap->source.name, 
+					scrap->source.line,
+					minscale + 1));
+			}
+		}
+	}
+
+	
   if (scrap->fsptr != NULL) {  
     fprintf(out->file, "current_scrap := \"%s@%s\";\n", scrap->name, 
       scrap->fsptr->full_name);
@@ -1290,6 +1428,9 @@ thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap,
       switch (obj->get_class_id()) {
         case TT_POINT_CMD:
           switch (((thpoint*)obj)->type) {
+            case TT_POINT_TYPE_DIMENSIONS:
+              obj->tags &= ~TT_2DOBJ_TAG_VISIBILITY_ON;
+              break;
             case TT_POINT_TYPE_BEDROCK:
             case TT_POINT_TYPE_SAND:
             case TT_POINT_TYPE_RAFT:
@@ -1590,12 +1731,18 @@ thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap,
         fprintf(out->file,"%s((%.2f,%.2f));\n",
           out->symset->get_mp_macro(macroid),
           thxmmxst(out, slp->stx, slp->sty));
+        if (out->layout->is_debug_stationnames() && (slp->station_name.name != NULL)) {
+          dbg_stnms.appspf("p_label.urt(btex %s etex, (%.2f, %.2f), 0.0, 7);\n",
+            utf2tex(slp->station_name.name), 
+            thxmmxst(out, slp->stx, slp->sty));
+        }
       }
     }
     slp = slp->next_item;
   }
   
   // potom zo scrapu
+  bool expstation;
   obj = scrap->ls2doptr;
   if (outline_mode) obj = NULL;
   while (obj != NULL) {
@@ -1603,9 +1750,26 @@ thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap,
       switch (obj->get_class_id()) {
         case TT_POINT_CMD:
           if (((thpoint*)obj)->type == TT_POINT_TYPE_STATION) {
-            if (obj->export_mp(noout)) {
-              thexpmap_export_mp_bgif;
-              obj->export_mp(out);
+            ptp = (thpoint *) obj;
+            expstation = true;
+            if (ptp->station_name.id != 0) {
+              thdb1ds * station = &(thdb.db1d.station_vec[ptp->station_name.id - 1]);
+              if (station->flags & TT_STATIONFLAG_UNDERGROUND)
+                typid = SYMP_CAVESTATION;
+              else
+                typid = SYMP_SURFACESTATION;
+              expstation = out->symset->assigned[typid];
+            }
+            if (expstation) {
+              if (obj->export_mp(noout)) {
+                thexpmap_export_mp_bgif;
+                obj->export_mp(out);
+                if (out->layout->is_debug_stationnames() && (ptp->station_name.name != NULL)) {
+                  dbg_stnms.appspf("p_label.urt(btex %s etex, (%.2f, %.2f), 0.0, 7);\n",
+                    utf2tex(ptp->station_name.name), 
+                    thxmmxst(out, ptp->point->xt, ptp->point->yt));
+                }
+              }
             }
           }
           break;
@@ -1677,8 +1841,8 @@ thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap,
                 fprintf(out->file,",");
                 lp->export_nextcp_mp(out);
                 thdb.buff_enc.guarantee(4096);
-                sprintf(thdb.buff_enc.get_buffer(),"%.0f",lp->rsize);
-                fprintf(out->file,",btex \\thwallaltitude %s etex);\n",utf2tex(thdb.buff_enc.get_buffer()));
+                //sprintf(thdb.buff_enc.get_buffer(),"%.0f",lp->rsize - out->layout->goz);
+                fprintf(out->file,",btex \\thwallaltitude %s etex);\n",utf2tex(out->layout->units.format_length(lp->rsize - out->layout->goz)));
 //                fprintf(out->file,",\"%.0f\");\n",lp->rsize);
               }
               lp = lp->nextlp;
@@ -1712,7 +1876,24 @@ thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap,
     DO_NOT_EXPORT_LABEL:
     obj = obj->pscrapoptr;
   }
+
+  if (out->layout->is_debug_stationnames() && (strlen(dbg_stnms.get_buffer()) > 0)) {
+    thexpmap_export_mp_bgif;
+    fprintf(out->file, "%s", dbg_stnms.get_buffer());
+  }
   
+  // nakoniec scrap name, ak mame zapnuty dany debug mod
+  if (out->layout->is_debug_scrapnames()) {
+    thexpmap_export_mp_bgif;
+    thdb2dpt tmppt;
+    tmppt.xt = (scrap->lxmin + scrap->lxmax) / 2.0;
+    tmppt.yt = (scrap->lymin + scrap->lymax) / 2.0;
+    thdb.buff_tmp = utf2tex(scrap->name);
+    fprintf(out->file,"p_label(btex \\thlargesize %s etex,",thdb.buff_tmp.get_buffer());
+    tmppt.export_mp(out);
+    fprintf(out->file,",0.0,6);\n");
+  }
+
 
   if (somex) {
     fprintf(out->file,"endfig;\n");  
@@ -1735,6 +1916,7 @@ thexpmap_xmps thexpmap::export_mp(thexpmapmpxs * out, class thscrap * scrap,
 }
 
 #define tmp_alpha 0.75
+#define tmp_set_color(r,g,b) cR = r; cG = g; cB = b
 
 void thexpmap::export_pdf_set_colors(class thdb2dxm * maps, class thdb2dprj * prj)
 {
@@ -1818,10 +2000,13 @@ void thexpmap::export_pdf_set_colors(class thdb2dxm * maps, class thdb2dprj * pr
       clrec.G = cG; 
       clrec.B = cB;
       opacity_correction(clrec.R, clrec.G, clrec.B);
-      sprintf(tmpb.get_buffer(), "%.0f", curz);
-      clrec.name = utf2tex(tmpb.get_buffer());
-      clrec.name += "\\thinspace ";
-      clrec.name += utf2tex(thT("units m",layout->lang));
+      //sprintf(tmpb.get_buffer(), "%.0f", curz - this->layout->goz);
+      clrec.texname = utf2tex(this->layout->units.format_length(curz - this->layout->goz));
+      clrec.texname += "\\thinspace ";			
+      clrec.texname += utf2tex(this->layout->units.format_i18n_length_units());
+      clrec.name = this->layout->units.format_length(curz - this->layout->goz);
+      clrec.name += " ";			
+      clrec.name += this->layout->units.format_i18n_length_units();
       COLORLEGENDLIST.insert(COLORLEGENDLIST.end(), clrec);
     }
   }
@@ -1847,7 +2032,8 @@ void thexpmap::export_pdf_set_colors(class thdb2dxm * maps, class thdb2dprj * pr
               // vsetkym scrapom v kazdej priradi farbu
               if (firstmapscrap) {
                 thset_color(0, (double) (nmap - cmn), (double) nmap, cR, cG, cB);
-                clrec.name = utf2tex(strlen(cmap->map->title) > 0 ? cmap->map->title : cmap->map->name);
+                clrec.texname = utf2tex(strlen(cmap->map->title) > 0 ? cmap->map->title : cmap->map->name);
+                clrec.name = (strlen(cmap->map->title) > 0 ? cmap->map->title : cmap->map->name);
                 alpha_correction(tmp_alpha, cR, cG, cB);
                 clrec.R = cR;
                 clrec.G = cG; 
@@ -1862,6 +2048,22 @@ void thexpmap::export_pdf_set_colors(class thdb2dxm * maps, class thdb2dprj * pr
               cs->B = cB;
 //              thprintf("%s@%s->%.2f,%.2f,%.2f\n",cs->name,cs->fsptr->full_name,cs->R,cs->G,cs->B);
             break;
+            case TT_LAYOUT_CCRIT_SCRAP:
+              // vsetkym scrapom v kazdej priradi farbu
+							switch (cmn % 6) {
+								case 0:  tmp_set_color(1.0, 0.5, 0.5); break;
+								case 1:  tmp_set_color(0.5, 1.0, 0.5); break;
+								case 2:  tmp_set_color(0.5, 0.5, 1.0); break;
+								case 3:  tmp_set_color(1.0, 1.0, 0.0); break;
+								case 4:  tmp_set_color(0.0, 1.0, 1.0); break;
+								default: tmp_set_color(1.0, 0.0, 1.0); break;
+							}
+              alpha_correction(tmp_alpha, cR, cG, cB);
+              cs->R = cR;
+              cs->G = cG;
+              cs->B = cB;
+              cmn++;
+            break;
             case TT_LAYOUT_CCRIT_ALTITUDE:
               // priradi farbu podla (z - min) z (max - min)
               if (!thisnan(curz)) {
@@ -1885,7 +2087,483 @@ void thexpmap::export_pdf_set_colors(class thdb2dxm * maps, class thdb2dprj * pr
 }
 
 
+void thexpmap::export_xvi(class thdb2dprj * prj)
+{
+  char * fnm;  
+  if (this->outpt_def)
+    fnm = this->outpt;
+  else
+    fnm = "cave.xvi";
+    
+  switch (prj->type) {
+    case TT_2DPROJ_PLAN:
+    case TT_2DPROJ_ELEV:
+    case TT_2DPROJ_EXTEND:
+      break;
+    default:
+      thwarning(("projection type not supported in XVI export"));
+      return;
+  }
+  
+  if (thdb.db1d.station_vec.size() == 0) {
+    thwarning(("no data to export"));
+    return;
+  }
+  
+#ifdef THDEBUG
+  thprintf("\n\nwriting %s\n", fnm);
+#else
+  thprintf("writing %s ... ", fnm);
+  thtext_inline = true;
+#endif 
+      
+  unsigned long i, nst = thdb.db1d.station_vec.size(), nsh = thdb.db1d.leg_vec.size();
+  thdb1ds * cs;
+  thdataleg * cl;
+  lxVec * stvec;
+  bool * isexp;
+  FILE * pltf;
 
+  pltf = fopen(fnm,"w");
+     
+  if (pltf == NULL) {
+    thwarning(("can't open %s for output",fnm))
+    return;
+  }
+
+  isexp = new bool [nst];
+  unsigned long nstvec;
+  if (prj->type == TT_2DPROJ_EXTEND)
+    nstvec = 2 * nsh;
+  else
+    nstvec = nst;
+  stvec = new lxVec [nstvec == 0 ? 1 : nstvec];
+
+  layoutnan(gxs, 1.0);
+  layoutnan(gys, 1.0);
+  layoutnan(gzs, 1.0);
+  layoutnan(gox, 0.0);
+  layoutnan(goy, 0.0);
+  layoutnan(goz, 0.0);
+  
+  double xx, yx, zx, xy, yy, zy, xmin = thnan, xmax = thnan, ymin = thnan, ymax = thnan, cx, cy, sf;
+  double shx, shy, gxs, gxo, gyo, gxoo, gyoo, alpha;
+  long gxn, gyn;
+  
+  // scale factor: assume 100dpi
+  sf = 3937.00787402 * this->layout->scale;
+  gxs = this->layout->gxs;
+  fprintf(pltf,"set XVIgrids {%.1f m}\n", gxs);
+  gxs *= sf;
+    
+  switch (prj->type) {
+    case TT_2DPROJ_ELEV:
+    case TT_2DPROJ_EXTEND:
+      if (thisnan(prj->pp1))
+        alpha = 0.0;
+      else
+        alpha = prj->pp1;
+      xx = sf * cos(alpha / 180.0 * THPI);  xy = 0.0;
+      yx = sf * -sin(alpha / 180.0 * THPI);  yy = 0.0;
+      zx = 0.0; zy = sf;
+      gxo = sf * this->layout->gox * xx + this->layout->goy * yx;
+      gyo = sf * this->layout->goz;
+      break;
+    default:
+      xx = sf; xy = 0.0;
+      yx = 0.0; yy = sf;
+      zx = 0.0; zy = 0.0;
+      gxo = sf * this->layout->gox;
+      gyo = sf * this->layout->goy;
+  }
+  
+  // select stations from selected surveys
+  long survid = -1;
+  unsigned long ff, tt;
+
+#define check_cxy_minmax() \
+  if (thisnan(xmin)) { \
+    xmin = cx; \
+    ymin = cy; \
+    xmax = cx; \
+    ymax = cy; \
+  } else { \
+    if (xmin > cx) xmin = cx; \
+    if (ymin > cy) ymin = cy; \
+    if (xmax < cx) xmax = cx; \
+    if (ymax < cy) ymax = cy; \
+  }
+
+
+  for(i = 0; i < nst; i++) {
+    cs = &(thdb.db1d.station_vec[i]);
+    if (cs->survey->is_selected()) {
+      if ((survid > -2) && (long(cs->survey->id) != survid)) {
+        if (survid == -1)
+          survid = (long) cs->survey->id;
+        else
+          survid = -2;
+      }
+      isexp[i] = true;
+      cx = cs->x * xx + cs->y * yx + cs->z * zx;
+      cy = cs->x * xy + cs->y * yy + cs->z * zy;
+      check_cxy_minmax();
+      stvec[i].x = cx;
+      stvec[i].y = cy;
+      stvec[i].z = 0.0;
+    } else {
+      isexp[i] = false;
+    }
+  }
+
+  if (prj->type == TT_2DPROJ_EXTEND) {
+    xmin = thnan;
+    for(i = 0; i < nsh; i++) {
+      cl = thdb.db1d.leg_vec[i].leg;
+      ff = cl->from.id - 1;
+      tt = cl->to.id - 1;
+
+      if (isexp[ff] && isexp[tt] && ((cl->extend & TT_EXTENDFLAG_IGNORE) == 0)) {
+
+        cs = &(thdb.db1d.station_vec[ff]);
+        cx = sf * cl->fxx;
+        cy = sf * cs->z;
+        check_cxy_minmax();
+        stvec[2*i].x = cx;
+        stvec[2*i].y = cy;
+        stvec[2*i].z = 0.0;
+
+        cs = &(thdb.db1d.station_vec[tt]);
+        cy = sf * cs->z;
+        cx = sf * cl->txx;
+        check_cxy_minmax();
+        stvec[2*i+1].x = cx;
+        stvec[2*i+1].y = cy;
+        stvec[2*i+1].z = 0.0;
+      }
+    }
+  }
+  
+  shx = (xmin + xmax) / 2.0;
+  shy = (ymin + ymax) / 2.0;
+  xmin -= shx; xmax -= shx; gxo -= shx;
+  ymin -= shy; ymax -= shy; gyo -= shy;
+
+  thbuffer stname;
+  thsurvey * css;
+  fprintf(pltf,"set XVIstations {\n");
+  for(i = 0; i < nstvec; i++) {
+
+# define export_xvi_station(i,j)  \
+    if (isexp[(i)]) { \
+      cs = &(thdb.db1d.station_vec[(i)]); \
+      stname = cs->name; \
+      css = cs->survey; \
+      if (survid == -2) { \
+        while ((css != NULL) && (css->is_selected())) { \
+          if (css->id == cs->survey->id) \
+            stname += "@"; \
+          else \
+            stname += "."; \
+          stname += css->name; \
+          css = css->fsptr; \
+        } \
+      } \
+      stvec[(j)].x -= shx; \
+      stvec[(j)].y -= shy; \
+      fprintf(pltf,"  {%12.2f %12.2f %s}\n", stvec[(j)].x, stvec[(j)].y, stname.get_buffer()); \
+    }
+    if (prj->type == TT_2DPROJ_EXTEND) {
+      if (i % 2 == 0) {
+        cl = thdb.db1d.leg_vec[i / 2].leg;
+        ff = cl->from.id - 1;
+        tt = cl->to.id - 1;
+        export_xvi_station(ff,i);
+        export_xvi_station(tt,i+1);
+      }
+    } else
+      export_xvi_station(i,i);
+  }
+  fprintf(pltf,"}\n");
+  
+  // calculate grid
+  double goverlap = 0.1 * hypot(xmax - xmin, ymax - ymin);
+  thset_grid(gxo, gxs, xmin - goverlap, xmax + goverlap, gxoo, gxn);
+  thset_grid(gyo, gxs, ymin - goverlap, ymax + goverlap, gyoo, gyn);
+  fprintf(pltf,"set XVIgrid  {%g %g %g 0.0 0.0 %g %ld %ld}\n", gxoo, gyoo, gxs, gxs, gxn+1, gyn+1);
+  
+  // export shots
+  fprintf(pltf,"set XVIshots {\n");
+  lxVec vff, vf1, vf2, vtt, vt1, vt2, rvec;
+  double fl, fr, tl, tr;
+  bool vertical;
+  lxVecLimits vlim;
+  for(i = 0; i < nsh; i++) {
+    cl = thdb.db1d.leg_vec[i].leg;
+    ff = cl->from.id - 1;
+    tt = cl->to.id - 1;
+    if (prj->type == TT_2DPROJ_EXTEND) {
+      vff = stvec[2*i];
+      vtt = stvec[2*i+1];
+    } else {
+      vff = stvec[ff];
+      vtt = stvec[tt];
+    }
+    if (isexp[ff] && isexp[tt] && ((prj->type != TT_2DPROJ_EXTEND) || ((cl->extend & TT_EXTENDFLAG_IGNORE) == 0))) {
+      fprintf(pltf,"  {%12.2f %12.2f %12.2f %12.2f", vff.x, vff.y, vtt.x, vtt.y);
+      // calculate and export LRUD if needed
+      if (cl->walls != TT_FALSE) {
+        if ((vff.x == vtt.x) && (vff.y == vtt.y)) {
+          vlim.valid = false;
+          vlim.Add(-sf * cl->from_left,-sf * cl->from_down,0.0);
+          vlim.Add(sf * cl->from_right, sf * cl->from_up  ,0.0);
+          vlim.Add(-sf * cl->to_left,-sf * cl->to_down,0.0);
+          vlim.Add(sf * cl->to_right, sf * cl->to_up  ,0.0);
+          vf1 = vff + vlim.min;
+          vf2 = vff + lxVec(vlim.min.x, vlim.max.y, 0.0);
+          vt2 = vff + vlim.max;
+          vt1 = vff + lxVec(vlim.max.x, vlim.min.y, 0.0);
+        } else {
+          vertical = false;
+          switch (prj->type) {
+            case TT_2DPROJ_ELEV:
+            case TT_2DPROJ_EXTEND:
+              if (fabs(cl->total_gradient) < cl->vtresh)
+                vertical = true;
+              if ((vertical) || (vff.x < vtt.x) || ((vff.x == vtt.x) && (vff.y < vtt.y))) {
+                fl = sf * cl->from_up; 
+                fr = sf * cl->from_down; 
+                tl = sf * cl->to_up;
+                tr = sf * cl->to_down;
+              } else {
+                fl = sf * cl->from_down; 
+                fr = sf * cl->from_up; 
+                tl = sf * cl->to_down;
+                tr = sf * cl->to_up;
+              }
+              break;
+            default:
+              fl = sf * cl->from_left; 
+              fr = sf * cl->from_right; 
+              tl = sf * cl->to_left;
+              tr = sf * cl->to_right;
+          }
+          if (vertical) {
+            rvec = lxVec(0.0, 1.0, 0.0);
+          } else {
+            rvec = vtt - vff;
+            rvec.Normalize();
+            rvec = rvec.Rotated(90.0,0.0);
+          }
+          vf1 = vff + fl * rvec;
+          vf2 = vff - fr * rvec;
+          vt1 = vtt + tl * rvec;
+          vt2 = vtt - tr * rvec;
+        }
+        fprintf(pltf," %12.2f %12.2f %12.2f %12.2f %12.2f %12.2f %12.2f %12.2f", 
+          vf1.x, vf1.y, vt1.x, vt1.y, vt2.x, vt2.y, vf2.x, vf2.y);
+      }
+      fprintf(pltf,"}\n");
+    }
+  }
+  fprintf(pltf,"}\n");
+
+  delete [] isexp;
+  delete [] stvec;
+  fclose(pltf);
+  
+#ifdef THDEBUG
+#else
+  thprintf("done\n");
+  thtext_inline = false;
+#endif
+}
+
+
+void thexpmap::export_uni(class thdb2dxm * maps, class thdb2dprj * prj)
+{
+
+  if (maps == NULL) {
+    thwarning(("%s [%d] -- no selected projection data -- %s",
+      this->src.name, this->src.line, this->projstr))
+    return;
+  }
+
+  char * fnm;  
+  if (this->outpt_def)
+    fnm = this->outpt;
+  else
+    fnm = "cave.3d";
+
+  img * pimg;
+  pimg = img_open_write(fnm, "cave", 1);
+  if (pimg == NULL) {
+    thwarning(("can't open %s for output",fnm))
+    return;
+  }
+  
+#ifdef THDEBUG
+  thprintf("\n\nwriting %s\n", fnm);
+#else
+  thprintf("writing %s ... ", fnm);
+  thtext_inline = true;
+#endif     
+
+  thdb2dxm * cmap = maps;
+  thdb2dxs * cbm;
+  thdb2dmi * cmi;
+
+  while (cmap != NULL) {
+    cbm = cmap->first_bm;
+    while (cbm != NULL) {
+      cmi = cbm->bm->last_item;
+      if (cbm->mode == TT_MAPITEM_NORMAL) {
+        while (cmi != NULL) {
+          if (cmi->type == TT_MAPITEM_NORMAL)
+						this->export_uni_scrap((FILE *) pimg, (thscrap *) cmi->object);
+          cmi = cmi->prev_item;  
+        }
+      }
+      cbm = cbm->next_item;
+    }
+    cmap = cmap->next_item;
+  }
+
+  img_close(pimg);
+    
+#ifdef THDEBUG
+#else
+  thprintf("done\n");
+  thtext_inline = false;
+#endif
+}
+
+
+void thexpmap_line_svx3d(img * pimg, thline * pln)
+{
+	thdb2dlp * plp = pln->first_point, * lp;
+	if (plp != NULL) 
+		lp = plp->nextlp;
+	else 
+		return;
+	bool move_to = true;
+	double t, tt, ttt, t_, tt_, ttt_, nx, ny, nz, px(0.0), py(0.0);
+	while (lp != NULL) {
+		if (lp->subtype != TT_LINE_SUBTYPE_INVISIBLE) {
+			if (move_to) {
+	      img_write_item(pimg, img_MOVE, 0, NULL, plp->point->xt, plp->point->yt, plp->point->zt);
+				px = plp->point->xt;
+				py = plp->point->yt;
+				move_to = false;
+			}
+			if ((lp->cp1 != NULL) && (lp->cp2 != NULL)) {
+				for(t = 0.05; t < 1.0; t += 0.05) {
+				  tt = t * t;
+					ttt = tt * t;
+					t_ = 1.0 - t;
+					tt_ = t_ * t_;
+					ttt_ = tt_ * t_;				
+					nx = ttt_ * plp->point->xt + 
+							 3.0 * t * tt_ * lp->cp1->xt + 
+							 3.0 * tt * t_ * lp->cp2->xt + 
+							 ttt * lp->point->xt;
+					ny = ttt_ * plp->point->yt + 
+							 3.0 * t * tt_ * lp->cp1->yt + 
+							 3.0 * tt * t_ * lp->cp2->yt + 
+							 ttt * lp->point->yt;
+					nz = t_ * plp->point->zt + t * lp->point->zt;
+					if (hypot(nx - px, ny - py) > 0.5) {
+			      img_write_item(pimg, img_LINE, 0, NULL, nx, ny, nz);
+						px = nx;
+						py = ny;
+					}
+				}			
+			}
+      img_write_item(pimg, img_LINE, 0, NULL, lp->point->xt, lp->point->yt, lp->point->zt);
+			px = lp->point->xt;
+			py = lp->point->yt;
+		} else {
+			move_to = true;
+		}
+		plp = lp;
+		lp = lp->nextlp;
+	}
+}
+
+
+void thexpmap::export_uni_scrap(FILE * out, class thscrap * scrap)
+{
+	
+	img * pimg;
+	pimg = (img *) out;
+	thbuffer stnbuff;
+	
+	double avx = 0.0, avy = 0.0, avz = 0.0, avn = 0.0;
+#define avadd(x,y,z) {avx	+= x; avy += y; avz += z; avn += 1.0;}
+	// export shots
+  thscraplp * slp;
+  slp = scrap->get_polygon();
+  while (slp != NULL) {
+    if (slp->lnio) {
+      img_write_item(pimg, img_MOVE, 0, NULL, slp->lnx1, slp->lny1, slp->lnz1);
+      img_write_item(pimg, img_LINE, img_FLAG_SURFACE, NULL, slp->lnx2, slp->lny2, slp->lnz2);
+    } else {
+			stnbuff = slp->station->survey->get_reverse_full_name();
+			stnbuff += ".";
+			stnbuff += slp->station->name;
+//			img_write_item(pimg, img_LABEL, 0, stnbuff, slp->stx,  slp->sty,  slp->stz);
+			avadd(slp->stx,  slp->sty,  slp->stz);
+		}
+    slp = slp->next_item;
+  }
+	
+	// export objects
+  th2ddataobject * obj;
+	thpoint * ppt;
+	thline * pln;
+  obj = scrap->ls2doptr;
+  while (obj != NULL) {
+		switch (obj->get_class_id()) {
+			case TT_POINT_CMD:
+				ppt = ((thpoint*)obj);
+	      switch (ppt->type) {
+	        case TT_POINT_TYPE_STATION:
+						if ((ppt->cpoint != NULL) && (ppt->cpoint->st != NULL)) {
+							stnbuff = ppt->cpoint->st->survey->get_reverse_full_name();
+							stnbuff += ".";
+							stnbuff += ppt->cpoint->st->name;
+//							img_write_item(pimg, img_LABEL, img_SFLAG_SURFACE, stnbuff, ppt->point->xt, ppt->point->yt, ppt->point->zt);
+							avadd(ppt->point->xt, ppt->point->yt, ppt->point->zt);
+						}
+  		      break;
+				}
+				break;
+		  case TT_LINE_CMD:
+				pln = ((thline*)obj);
+	      switch (pln->type) {
+					case TT_LINE_TYPE_WALL:
+						thexpmap_line_svx3d(pimg, pln);
+						thdb2dlp * lp = pln->first_point;
+						while (lp != NULL) {
+							avadd(lp->point->xt,lp->point->yt,lp->point->zt);
+							lp = lp->nextlp;
+						}
+						break;
+				}
+				break;
+    }
+    obj = obj->pscrapoptr;
+  }
+
+	if (avn > 0.0) {
+//		stnbuff = "SCRAP.";
+		stnbuff = scrap->fsptr->get_reverse_full_name();
+		stnbuff += ".";
+		stnbuff += scrap->name;
+		img_write_item(pimg, img_LABEL, 0, stnbuff, avx/avn, avy/avn, avz/avn);
+	}
+		
+}
 
 
 double thexpmap_quick_map_export_scale;
