@@ -37,6 +37,13 @@
 #include "thlang.h"
 #include <string.h>
 
+enum {
+  TTLDBG_JOINS = 1,
+  TTLDBG_STATIONS = 2,
+  TTLDBG_SCRAPNAMES = 4,
+  TTLDBG_STATIONNAMES = 8,
+};
+
 void thlayout_color::parse(char * str) {
   thsplit_words(&(thdb.mbuff_tmp), str);
   int nargs = thdb.mbuff_tmp.get_size(), sv;
@@ -154,6 +161,9 @@ thlayout::thlayout()
   this->def_opacity = false;
   this->opacity = 0.7;
 
+  this->def_map_header_bg = false;
+  this->map_header_bg = false;
+
   this->def_surface = false;
   this->surface = TT_LAYOUT_SURFACE_OFF;
   this->def_surface_opacity = false;
@@ -181,7 +191,7 @@ thlayout::thlayout()
   this->map_header_y = 0.0;
 
   this->def_debug = false;
-  this->debug = TT_LAYOUT_DEBUG_UNKNOWN;
+  this->debug = 0;
 
   
   this->def_max_explos = false;
@@ -201,6 +211,9 @@ thlayout::thlayout()
   this->def_lang = false;
   this->lang = THLANG_UNKNOWN;
   
+  this->def_units = false;
+  this->units = thdeflocale;
+  
   this->def_layers = false;
   this->layers = true;
 
@@ -668,8 +681,27 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
     case TT_LAYOUT_DEBUG:
       sv = thmatch_token(args[0],thtt_layout_debug);
       if (sv == TT_LAYOUT_DEBUG_UNKNOWN)
-        ththrow(("invalid debug switch -- %s",args[0]))
-      this->debug = sv;
+        ththrow(("invalid debug switch -- %s",args[0]))        
+      switch (sv) {
+        case TT_LAYOUT_DEBUG_ALL:
+          this->debug = TTLDBG_JOINS | TTLDBG_STATIONS | TTLDBG_SCRAPNAMES | TTLDBG_STATIONNAMES;
+          break;
+        case TT_LAYOUT_DEBUG_OFF:
+          this->debug = 0;
+          break;
+        case TT_LAYOUT_DEBUG_STATIONS:
+          this->debug |= TTLDBG_STATIONS;
+          break;
+        case TT_LAYOUT_DEBUG_JOINS:
+          this->debug |= TTLDBG_JOINS;
+          break;
+        case TT_LAYOUT_DEBUG_SCRAPNAMES:
+          this->debug |= TTLDBG_SCRAPNAMES;
+          break;
+        case TT_LAYOUT_DEBUG_STATIONNAMES:
+          this->debug |= TTLDBG_STATIONNAMES;
+          break;
+      }
       this->def_debug = true;
       break;
     
@@ -681,6 +713,11 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->def_lang = true;
       break;
     
+    case TT_LAYOUT_UNITS:
+			this->units.parse_units(args[0]);
+      this->def_units = true;
+      break;
+    
     case TT_LAYOUT_LAYERS:
       sv = thmatch_token(args[0],thtt_bool);
       if (sv == TT_UNKNOWN_BOOL)
@@ -688,6 +725,14 @@ void thlayout::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       this->layers = (sv == TT_TRUE);
       this->def_layers = true;
       break;
+
+    case TT_LAYOUT_MAP_HEADER_BG:
+      sv = thmatch_token(args[0],thtt_bool);
+      if (sv == TT_UNKNOWN_BOOL)
+        ththrow(("invalid map-header-bg switch -- %s",args[0]))
+      this->map_header_bg = (sv == TT_TRUE);
+      this->def_map_header_bg = true;
+      break;
     
     case TT_LAYOUT_OPACITY:
       thparse_double(sv,dum,args[0]);        
@@ -1014,9 +1059,15 @@ void thlayout::self_print_library() {
   thprintf("\tplayout->def_lang = %s;\n",(this->def_lang ? "true" : "false"));
   thprintf("\tplayout->lang = %s;\n",thlang_getcxxid(this->lang));
 
+  thprintf("\tplayout->def_units = %s;\n",(this->def_units ? "true" : "false"));
+  thprintf("\tplayout->units.units = %d;\n",this->units.units);
+
   thprintf("\tplayout->def_layers = %s;\n",(this->def_layers ? "true" : "false"));
   thprintf("\tplayout->layers = %s;\n",(this->layers ? "true" : "false"));
 
+  thprintf("\tplayout->def_map_header_bg = %s;\n",(this->def_map_header_bg ? "true" : "false"));
+  thprintf("\tplayout->map_header_bg = %s;\n",(this->map_header_bg ? "true" : "false"));
+
   thprintf("\tplayout->def_opacity = %s;\n",(this->def_opacity ? "true" : "false"));
   thprintf("\tplayout->opacity = %lg;\n",this->opacity);
 
@@ -1359,7 +1410,8 @@ void thlayout::export_pdftex(FILE * o, thdb2dprj * prj, char mode) {
             anyline = true;
         if ((!anylegend) && (strstr(ln->line, "\\formattedlegend") != NULL))
             anylegend = true;
-        thdecode(&(this->db->buff_enc), TT_ASCII, ln->line);
+        
+        thdecode_utf2tex(&(this->db->buff_enc), ln->line);
         fprintf(o, "%s\n", this->db->buff_enc.get_buffer());
       }
       ln = ln->next_line;
@@ -1618,9 +1670,15 @@ void thlayout::process_copy() {
 
       if (has_srcl(def_lang) && (srcl->lang != THLANG_UNKNOWN))
         this->lang = srcl->lang;
+
+      if (has_srcl(def_units))
+        this->units = srcl->units;
   
       if has_srcl(def_layers)
         this->layers = srcl->layers;
+
+      if has_srcl(def_map_header_bg)
+        this->map_header_bg = srcl->map_header_bg;
   
       if has_srcl(def_grid)
         this->grid = srcl->grid;
@@ -1684,6 +1742,7 @@ void thlayout::set_thpdf_layout(thdb2dprj * prj, double x_scale, double x_origin
   LAYOUT.page_numbering = this->pgsnum;
   LAYOUT.transparency = this->transparency;
   LAYOUT.OCG = this->layers;
+  LAYOUT.map_header_bg = this->map_header_bg;
   //TODO
   LAYOUT.map_grid = this->page_grid;
   LAYOUT.hsize = this->hsize * THM2PT;
@@ -1742,11 +1801,19 @@ void thlayout::set_thpdf_layout(thdb2dprj * prj, double x_scale, double x_origin
 
 
 bool thlayout::is_debug_stations() {
-  return ((this->debug == TT_LAYOUT_DEBUG_ALL) || (this->debug == TT_LAYOUT_DEBUG_STATIONS));
+  return ((this->debug & TTLDBG_STATIONS) != 0);
 }
 
 bool thlayout::is_debug_joins() {
-  return ((this->debug == TT_LAYOUT_DEBUG_ALL) || (this->debug == TT_LAYOUT_DEBUG_JOINS));
+  return ((this->debug & TTLDBG_JOINS) != 0);
+}
+
+bool thlayout::is_debug_scrapnames() {
+  return ((this->debug & TTLDBG_SCRAPNAMES) != 0);
+}
+
+bool thlayout::is_debug_stationnames() {
+  return ((this->debug & TTLDBG_STATIONNAMES) != 0);
 }
 
 std::list <thlayout_copy_src> thlayout_copy_src_list;
