@@ -33,6 +33,7 @@
 #include "thdataobject.h"
 #include "thlayoutln.h"
 #include "thsymbolset.h"
+#include "thlocale.h"
 #include <list>
 
 /**
@@ -86,6 +87,8 @@ enum {
   TT_LAYOUT_ROTATE = 2043,
   TT_LAYOUT_ENDCODE = 2044,
   TT_LAYOUT_COLOR_LEGEND = 2045,
+  TT_LAYOUT_MAP_HEADER_BG = 2046,
+	TT_LAYOUT_UNITS = 2047,
 };
 
 
@@ -155,6 +158,8 @@ enum {
   TT_LAYOUT_DEBUG_OFF,
   TT_LAYOUT_DEBUG_STATIONS,
   TT_LAYOUT_DEBUG_JOINS,
+  TT_LAYOUT_DEBUG_SCRAPNAMES,
+  TT_LAYOUT_DEBUG_STATIONNAMES,
   TT_LAYOUT_DEBUG_ALL,
   TT_LAYOUT_DEBUG_UNKNOWN,
 };
@@ -169,7 +174,9 @@ static const thstok thtt_layout_debug[] = {
   {"first", TT_LAYOUT_DEBUG_STATIONS},
   {"off", TT_LAYOUT_DEBUG_OFF},
   {"on", TT_LAYOUT_DEBUG_ALL},
+  {"scrap-names", TT_LAYOUT_DEBUG_SCRAPNAMES},
   {"second", TT_LAYOUT_DEBUG_JOINS},
+  {"station-names", TT_LAYOUT_DEBUG_STATIONNAMES},
   {NULL, TT_LAYOUT_DEBUG_UNKNOWN},
 };
 
@@ -310,6 +317,7 @@ enum {
   TT_LAYOUT_CCRIT_TOPODATE,
   TT_LAYOUT_CCRIT_EXPLODATE,
   TT_LAYOUT_CCRIT_MAP,
+  TT_LAYOUT_CCRIT_SCRAP,
 };
 
 static const thstok thtt_layout_ccrit[] = {
@@ -317,6 +325,7 @@ static const thstok thtt_layout_ccrit[] = {
 //  {"topo-date", TT_LAYOUT_CCRIT_TOPODATE},
 //  {"explo-date", TT_LAYOUT_CCRIT_EXPLODATE},
   {"map", TT_LAYOUT_CCRIT_MAP},
+  {"scrap", TT_LAYOUT_CCRIT_SCRAP},
   {NULL, TT_LAYOUT_CCRIT_UNKNOWN}
 };
 
@@ -384,6 +393,7 @@ static const thstok thtt_layout_opt[] = {
   {"legend-width",TT_LAYOUT_LEGEND_WIDTH},
   {"map-comment",TT_LAYOUT_DOC_COMMENT},
   {"map-header",TT_LAYOUT_MAP_HEADER},
+  {"map-header-bg",TT_LAYOUT_MAP_HEADER_BG},
   {"nav-factor",TT_LAYOUT_NAV_FACTOR},
   {"nav-size",TT_LAYOUT_NAV_SIZE},
   {"opacity",TT_LAYOUT_OPACITY},
@@ -407,6 +417,7 @@ static const thstok thtt_layout_opt[] = {
   {"symbol-show", TT_LAYOUT_SYMBOL_SHOW},
   {"title-pages",TT_LAYOUT_TITLE_PAGES},
   {"transparency",TT_LAYOUT_TRANSPARENCY},
+  {"units",TT_LAYOUT_UNITS},
   {NULL, TT_LAYOUT_UNKNOWN},
 };
 
@@ -467,7 +478,9 @@ class thlayout : public thdataobject {
   thlayoutln * first_line, * last_line;
   
   bool titlep, transparency, layers, pgsnum, lock, excl_pages, page_grid, 
-    explo_lens, topo_lens;
+    explo_lens, topo_lens, map_header_bg;
+		
+	thlocale units;
   
   bool def_grid_size, def_grid_origin, def_nav_factor, def_nav_size, 
     def_opacity, def_transparency, def_layers, def_base_scale,
@@ -477,10 +490,10 @@ class thlayout : public thdataobject {
     def_tex_lines, def_doc_title, def_doc_comment, def_doc_author, def_doc_subject,
     def_doc_keywords, def_excl_pages, def_grid, def_page_grid,
     def_legend, def_color_legend, def_legend_width, def_legend_columns, 
-    def_map_header, def_lang, def_scale_bar, 
+    def_map_header, def_lang, def_scale_bar, def_map_header_bg,
     redef_base_scale, def_max_explos, def_max_topos, def_max_cartos,
     def_max_copys, def_explo_lens, def_topo_lens, def_debug, def_surface,
-    def_surface_opacity;
+    def_surface_opacity, def_units;
     
   
   thlayout_copy_src * first_copy_src, * last_copy_src;
@@ -618,6 +631,8 @@ class thlayout : public thdataobject {
   
   bool is_debug_stations();
   bool is_debug_joins();
+  bool is_debug_scrapnames();
+  bool is_debug_stationnames();
   
   /**
    * set LAYOUT variable.
