@@ -32,6 +32,13 @@
 #include "thbuffer.h"
 #include "thmbuffer.h"
 
+
+enum {
+  TT_IMG_TYPE_UNKNOWN,
+  TT_IMG_TYPE_JPEG,
+  TT_IMG_TYPE_PNG,
+};
+
 /**
  * Token definition structure.
  */
@@ -294,12 +301,15 @@ bool th_is_extkeyword(char * str);
  
 void thdecode_c(thbuffer * dest, const char * src);
 
+void thdecode_tcl(thbuffer * dest, const char * src);
+
 
 /**
  * Decode string into TeX format.
  */
  
 void thdecode_tex(thbuffer * dest, const char * src);
+void thdecode_utf2tex(thbuffer * dest, const char * src);
 void thdecode_sql(thbuffer * dest, const char * src);
 
 
@@ -321,7 +331,7 @@ void thparse_altitude(char * src, double & altv, double & fixv);
  * Check image.
  */
 
-void thparse_image(char * fname, double & width, double & height, double & dpi);
+void thparse_image(char * fname, double & width, double & height, double & dpi, int & type);
 
 
 /**
@@ -339,6 +349,13 @@ void thset_grid(double gorigin, double gsize, double min, double max, double & q
 
 
 
+/**
+ * Convert therion string to xhtml.
+ */
+ 
+const char * thutf82xhtml(const char * src);
+
+
 #endif
 
 
