@@ -29,11 +29,13 @@
 #include <list>
 #include <map>
 #include <set>
+#include "thepsparse.h"
 
 using namespace std;
 
 struct scraprecord {
   string name,F,B,I,E,X,G,C,P;      // name + files
+  converted_data Fc, Bc, Ic, Ec, Xc, Gc;
   double S1,S2;                      // shift
   int layer,level,sect;             // Y, V, Z
   double F1,F2,F3,F4,                // bounding boxes
@@ -62,11 +64,13 @@ struct layerrecord {
 
 struct legendrecord {
   string name, fname, descr;
+  converted_data ldata;
+  unsigned idsym, idfig, idnum;
 };
 
 struct colorlegendrecord {
   double R, G, B;
-  string name;
+  string texname, name;
 };
 
 struct paired {
@@ -75,12 +79,14 @@ struct paired {
   paired();
 };
 
+paired rotatedaround(paired x,paired o, double th);
+
 struct layout {
   string excl_list,labelx,labely,
          doc_author,doc_keywords,doc_subject,doc_title,doc_comment,
          northarrow, scalebar;
   bool  excl_pages,title_pages,page_numbering,
-        transparency,map_grid,OCG; 
+        transparency,map_grid,OCG,map_header_bg; 
   double hsize,vsize,overlap,
         hgridsize, vgridsize,
         hgridorigin, vgridorigin, gridrot,
@@ -108,11 +114,14 @@ extern list<scraprecord> SCRAPLIST;
 extern list<legendrecord> LEGENDLIST;
 extern list<colorlegendrecord> COLORLEGENDLIST;
 extern layout LAYOUT;
+extern list<pattern> PATTERNLIST;
+extern list<converted_data> GRIDLIST;
+extern converted_data NArrow, ScBar;
 
 
 struct surfpictrecord {
-  char * filename;
-  double dx, dy, xx, xy, yx, yy;
+  char * filename, * type;
+  double dx, dy, xx, xy, yx, yy, width, height;
 };
 
 
