@@ -25,8 +25,6 @@
 ## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ## --------------------------------------------------------------------
 
-
-
 set xth(destroyed) 0
 set xth(prj,name) "therion"
 set xth(prj,title) "therion user interface"
@@ -37,12 +35,16 @@ set xth(gui,dbg) ".xth_dbg"
 set xth(gui,help) ".xth_help"
 set xth(gui,message) ".xthmsg"
 set xth(gui,minsize) {480 300}
+set xth(gui,balloons) 0
+set xth(gui,toolbar) 1
+set xth(te,template) {}
 
 set xth(kb_control) Control
 set xth(kb_meta) Meta
 set xth(gui,compshow) 0
 set xth(gui,compcmd) "therion"
 set xth(gui,auto_save) 0
+set xth(gui,check_update) 1
 
 set xth(encodings) { iso8859-1 iso8859-2 iso8859-5 iso8859-7 utf-8 }
 set xth(kbencodings) {utf-8 iso8859-1 iso8859-2 cp1250 macCentEuro unicode}
@@ -66,6 +68,7 @@ set xth(app,me,filetypes) {
 
 set xth(app,cp,filetypes) {    
   {{Therion config files}       {thconfig*}}    
+  {{Therion config files }       {.thcfg .thconfig}}    
   {{All files}       {*}}    
 }
 
@@ -76,45 +79,15 @@ set xth(app,mv,filetypes) {
 
 set xth(icmds) {survey}
 set xth(cmds) {scrap centerline grade line area map layout}
-set dfs {8s}                    
-set dfss {8s}                    
-set dfuf {8.2fx {-}}             
-set dfdf {+8.2fx {-}}            
-set dfcf {8.2fx {-}}             
-set dfccf {8.2f {-}}            
-set dfgf {{8.2fx} {up down -}} 
-set xth(datafmts) [list \
-  "unknown      $dfs" \
-  "station      $dfss" \
-  "from         $dfss" \
-  "to           $dfss" \
-  "compass      $dfuf" \
-  "backcompass  $dfuf" \
-  "bearing      $dfuf" \
-  "backbearing  $dfuf" \
-  "tape         $dfcf" \
-  "length       $dfcf" \
-  "count        $dfccf" \
-  "counter      $dfccf" \
-  "fromcount    $dfccf" \
-  "tocount      $dfccf" \
-  "fromcounter  $dfccf" \
-  "tocounter    $dfccf" \
-  "gradient     $dfgf" \
-  "clino        $dfgf" \
-  "backgradient $dfgf" \
-  "backclino    $dfgf" \
-  "depth        $dfdf" \
-  "fromdepth    $dfdf" \
-  "todepth      $dfdf" \
-  "depthchange  $dfdf" \
-  "dx           $dfcf" \
-  "dy           $dfcf" \
-  "dz           $dfcf" \
-  "northing     $dfcf" \
-  "easting      $dfcf" \
-  "altitude     $dfcf" \
-]
+
+set xth(datafmt,unknown) {4s}                    
+set xth(datafmt,station) {4s}                    
+set xth(datafmt,length) {6.2fx {-}}             
+set xth(datafmt,counter) {6.2f {-}}            
+set xth(datafmt,bearing) {5.1fx {-}}             
+set xth(datafmt,gradient) {{5.1fx} {up down -}} 
+set xth(datafmt,depth) {6.2fx {-}}            
+set xth(datafmt,dimensions) {4.1fx {-}}             
 
 set xth(gui,initdir) [pwd]
 set xth(app,active) ""
@@ -152,6 +125,14 @@ set xth(gui,selbg) darkBlue
 set xth(gui,etabsize) 2
 set xth(gui,controlk) Ctrl
 
+set xth(gui,xvi_grid_clr) #00D0D0
+set xth(gui,xvi_walls_fptn) gray12
+set xth(gui,xvi_walls_fclr) gray80
+set xth(gui,xvi_walls_oclr) gray60
+set xth(gui,xvi_shot_clr) gray50
+set xth(gui,xvi_station_fclr) black
+set xth(gui,xvi_station_oclr) black
+
 set xth(gui,me,nozoom) 1
 
 destroy .def
